import StringIO
import unittest
import os
import tempfile
import time
import shutil
import subprocess
import hepdata_converter
from hepdata_converter import convert
from hepdata_converter.testsuite import insert_path, insert_data_as_str
from hepdata_converter.testsuite.test_writer import WriterTestSuite


class CSVWriterTestCase(WriterTestSuite):

    @insert_path('yaml_full')
    def setUp(self, submission_filepath):
        super(CSVWriterTestCase, self).setUp()
        self.submission_filepath = submission_filepath

    @insert_data_as_str('csv/table_1.csv')
    def test_csvwriter_options(self, table_1_content):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 'Table 1',
                                                                 'pack': True})

        self.assertMultiLineAlmostEqual(table_1_content, csv_content)

    @insert_data_as_str('csv/table_9.csv')
    def test_2_qualifiers_2_iv_pack(self, table_9_content):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 'Table 9',
                                                                 'pack': True})

        self.assertMultiLineAlmostEqual(table_9_content, csv_content)

    @insert_data_as_str('csv/table_1.csv')
    @insert_data_as_str('csv/table_9.csv')
    def test_multiple_tables_pack(self, table_1_content, table_9_content):
        convert(self.submission_filepath, self.current_tmp, options={'input_format': 'yaml',
                                                                     'output_format': 'csv',
                                                                     'pack': True})

        with open(os.path.join(self.current_tmp, 'Table 1.csv'), 'r') as f:
            self.assertMultiLineAlmostEqual(table_1_content, f.read())

        with open(os.path.join(self.current_tmp, 'Table 9.csv'), 'r') as f:
            self.assertMultiLineAlmostEqual(table_9_content, f.read())

    @insert_data_as_str('csv/table_9_unpacked.csv')
    def test_2_qualifiers_2_iv_unpack(self, table_9_content):
        csv_content = convert(self.submission_filepath, options={'input_format': 'yaml',
                                                         'output_format': 'csv',
                                                         'table': 'Table 9',
                                                         'separator': ';',
                                                         'pack': False})
        self.assertMultiLineAlmostEqual(table_9_content, csv_content)

    @insert_data_as_str('csv/table_9_unpacked.csv')
    def test_cli(self, table_9_content):
        csv_filepath = os.path.join(self.current_tmp, 'tab.csv')
        hepdata_converter._main(['--output-format', 'csv', '--table', 'Table 9', '--separator', ';', self.submission_filepath,
                                 csv_filepath])

        with open(csv_filepath, 'r') as csv_file:
            self.assertEqual(table_9_content, csv_file.read())

    @insert_data_as_str('csv/table_1.csv')
    @insert_data_as_str('csv/table_9_unpacked_comma.csv')
    def test_no_dir_output(self, table_1_content, table_9_content):
        csv_filepath = os.path.join(self.current_tmp, 'csv_dir')
        hepdata_converter._main(['--output-format', 'csv', '--separator', ',', self.submission_filepath,
                                 csv_filepath])

        self.assertTrue(os.path.exists(csv_filepath))
        csv_1 = os.path.join(csv_filepath, 'Table 1.csv')
        self.assertTrue(os.path.exists(csv_1))
        with open(csv_1, 'r') as csv_file:
            self.assertMultiLineAlmostEqual(table_1_content.strip(), csv_file.read().strip())

        csv_2 = os.path.join(csv_filepath, 'Table 9.csv')
        self.assertTrue(os.path.exists(csv_2))
        with open(csv_2, 'r') as csv_file:
            self.assertMultiLineAlmostEqual(table_9_content.strip(), csv_file.read().strip())

    @insert_path('oldhepdata/sample.input')
    @insert_path('csv/full')
    def test_diroutput(self, oldhepdata_path, csv_path):
        hepdata_converter._main(['-i', 'oldhepdata',
                                 '-o', 'csv',
                                 oldhepdata_path, self.current_tmp])

        self.assertDirsEqual(self.current_tmp, csv_path, file_content_parser=lambda x: x)



import abc
import copy
import csv
from math import sqrt
import os
from hepdata_converter.common import OptionInitMixin, Option
from hepdata_converter.writers import Writer
import abc


class ObjectWrapper(object):
    __metaclass__ = abc.ABCMeta
    accept_alphanumeric = False
    # :core_object: bool if set to false object of this class will be created as an *extra* additionally to the first
    # object of class with ```core_object``` set to True
    core_object = True

    @classmethod
    def is_number_var(cls, *variables):
        for variable in variables:
            for element in variable['values']:
                if 'value' in element and isinstance(element['value'], (str, unicode)):
                    return False
        return True
    @classmethod
    def sanitize_name(cls, name):
        return name.replace(' ', '_').replace('/', '_')

    @classmethod
    def is_value_var(cls, variable):
        return cls._is_attr_variable('value', variable)

    @classmethod
    def is_range_var(cls, variable):
        return cls._is_attr_variable('high', variable) and cls._is_attr_variable('low', variable)

    @classmethod
    def has_errors(cls, variable):
        return cls._is_attr_variable('errors', variable)

    @classmethod
    def _is_attr_variable(cls, attr_name, variable):
        for element in variable['values']:
            if attr_name not in element:
                return False
        return True

    def __init__(self, independent_variable_map, dependent_variable, dependent_variable_index):
        self.xval = []
        self.yval = []
        self.xerr_minus = []
        self.xerr_plus = []
        self.yerr_minus = []
        self.yerr_plus = []
        self.independent_variables = list(independent_variable_map)
        self.independent_variable_map = independent_variable_map
        self.dependent_variable = dependent_variable
        self.dependent_variable_index = dependent_variable_index

    @classmethod
    def match(cls, independent_variables_map, dependent_variable):
        if not cls.accept_alphanumeric and (len(filter(lambda x: x is False, [cls.is_number_var(var) for var in independent_variables_map])) > 0 or not cls.is_number_var(dependent_variable)):
            return False
        if len(independent_variables_map) == 0 or len(independent_variables_map[0]) == 0:
            return False
        return True

    @classmethod
    def match_and_create(cls, independent_variables_map, dependent_variable, dependent_variable_index):
        if cls.match(independent_variables_map, dependent_variable):
            return cls(independent_variables_map, dependent_variable, dependent_variable_index).create_objects()
        return []

    def calculate_total_errors(self):
        for independent_variable in self.independent_variable_map:
            xerr_minus = []
            self.xerr_minus.append(xerr_minus)
            xerr_plus = []
            self.xerr_plus.append(xerr_plus)
            xval = []
            self.xval.append(xval)
            ArrayWriter.calculate_total_errors(independent_variable, xerr_minus, xerr_plus, xval)
        ArrayWriter.calculate_total_errors(self.dependent_variable, self.yerr_minus, self.yerr_plus, self.yval)

    @abc.abstractmethod
    def create_objects(self):
        pass


class ObjectFactory(object):
    def __init__(self, class_list, independent_variables, dependent_variables):
        self.class_list = class_list
        self.map = {}
        self.independent_variables = independent_variables
        self.dependent_variables = dependent_variables
        for variable_index in xrange(len(dependent_variables)):
            self.map[variable_index] = list(independent_variables)

    def get_next_object(self):
        for dependent_variable_index in xrange(len(self.dependent_variables)):
            auxiliary_object_created = False
            for class_wrapper in self.class_list:
                # if auxiliary object was already created for this variable, don't create more
                # (eg. after creating TH3 (which does not consume variables) don't create TH2 & TH1,
                # but go for best fit of "core" objects eg. TGraph

                if not class_wrapper.core_object and auxiliary_object_created:
                    continue

                objects = class_wrapper.match_and_create(self.map[dependent_variable_index], self.dependent_variables[dependent_variable_index], dependent_variable_index)
                if objects and not class_wrapper.core_object:
                    auxiliary_object_created = True

                for obj in objects:
                    yield obj


class ArrayWriter(Writer):
    __metaclass__ = abc.ABCMeta

    @staticmethod
    def calculate_total_errors(variable, min_errs, max_errs, values):
        for entry in variable['values']:
            if 'value' in entry:
                values.append(entry['value'])
                if 'errors' in entry:
                    errors_min = 0.0
                    errors_max = 0.0
                    for error in entry['errors']:
                        if 'asymerror' in error:
                            errors_min += pow(error['asymerror']['minus'], 2)
                            errors_max += pow(error['asymerror']['plus'], 2)
                        elif 'symerror' in error:
                            errors_min += pow(error['symerror'], 2)
                            errors_max += pow(error['symerror'], 2)
                    min_errs.append(sqrt(errors_min))
                    max_errs.append(sqrt(errors_max))
                else:
                    min_errs.append(0.0)
                    max_errs.append(0.0)
            else:
                middle_val = (entry['high'] - entry['low']) * 0.5 + entry['low']
                values.append(middle_val)
                min_errs.append(entry['high'] - middle_val)
                max_errs.append(middle_val - entry['low'])

    @classmethod
    def options(cls):
        options = super(ArrayWriter, cls).options()
        options['table'] = Option('table', 't', required=False, variable_mapping='table_id', default=None,
                                  help=('Specifies which table should be exported, if not specified all tables will be exported '
                                        '(in this case output must be a directory, not a file)'))

        return options

    def __init__(self, *args, **kwargs):
        kwargs['single_file_output'] = True
        super(ArrayWriter, self).__init__(*args, **kwargs)
        self.tables = []
        self.extension = None

    @abc.abstractmethod
    def _write_table(self, data_out, table):
        pass

    def _get_tables(self, data_in):
        # get table to work on
        if self.table_id is not None:
            if isinstance(self.table_id, int):
                self.tables.append(data_in.get_table(id=self.table_id))
            else:
                try:
                    tab = data_in.get_table(file=self.table_id)
                except IndexError:
                    tab = data_in.get_table(name=self.table_id)

                self.tables.append(tab)
        else:
            self.tables = data_in.tables

    def _prepare_outputs(self, data_out, outputs):
        if isinstance(data_out, str) or isinstance(data_out, unicode):
            self.file_emulation = True
            if len(self.tables) == 1:
                f = open(data_out, 'w')
                outputs.append(f)
            # data_out is a directory
            else:
                # create output dir if it doesn't exist
                self.create_dir(data_out)
                for table in self.tables:
                    outputs.append(open(os.path.join(data_out, table.name + '.' + self.extension), 'w'))
        # multiple tables - require directory
        elif len(self.tables) > 1 and not (isinstance(data_out, str) or isinstance(data_out, unicode)):
            raise ValueError("Multiple tables, output must be a directory")
        else:
            outputs.append(data_out)

    def write(self, data_in, data_out, *args, **kwargs):
        """

        :param data_in:
        :type data_in: hepconverter.parsers.ParsedData
        :param data_out: filelike object
        :type data_out: file
        :param args:
        :param kwargs:
        """
        self._get_tables(data_in)

        self.file_emulation = False
        outputs = []

        self._prepare_outputs(data_out, outputs)

        for i in xrange(len(self.tables)):
            data_out = outputs[i]
            table = self.tables[i]

            self._write_table(data_out, table)

            if self.file_emulation:
                data_out.close()

    @classmethod
    def _extract_independent_variables(cls, table, headers, data, qualifiers_marks):
        for independent_variable in table.independent_variables:
            name = independent_variable['header']['name']
            if 'units' in independent_variable['header']:
                name += ' IN %s' % independent_variable['header']['units']
            headers.append(name)
            x_data_low = []
            x_data_high = []
            for value in independent_variable['values']:

                if 'high' in value and 'low' in value:
                    x_data_low.append(value['low'])
                    x_data_high.append(value['high'])
                else:
                    x_data_low.append(value['value'])
                    x_data_high.append(value['value'])

            data.append(x_data_low)
            if x_data_high != x_data_low:
                data.append(x_data_high)
                header = headers[-1]
                headers[-1] = header + ' LOW'
                headers.append(header + ' HIGH')
                qualifiers_marks.append(False)

    @classmethod
    def _parse_dependent_variable(cls, dependent_variable, headers, qualifiers, qualifiers_marks, data):
        units = ''
        if 'units' in dependent_variable['header']:
            units = ' IN %s' % dependent_variable['header']['units']
        headers.append(dependent_variable['header']['name'] + units)

        qualifiers_marks.append(True)

        # peek at first value and create empty lists
        y_order = []
        y_data = {'values': []}
        y_order.append(y_data['values'])
        # :TODO: GET ALL ERRORS FROM ALL ENTRIES
        for error in dependent_variable['values'][0].get('errors', []):
            headers.append(error.get('label', 'stat') + ' +')
            qualifiers_marks.append(False)
            headers.append(error.get('label', 'stat') + ' -')
            qualifiers_marks.append(False)

            plus = []
            y_data[error.get('label', 'stat')+'_plus'] = plus
            y_order.append(plus)
            minus = []
            y_data[error.get('label', 'stat')+'_minus'] = minus
            y_order.append(minus)

        for value in dependent_variable['values']:
            y_data['values'].append(value['value'])
            # :TODO: HANDLE SITUATION WHERE ENTRY HAS PARITAL ERRORS (NOT ALL OF THEM)
            if 'errors' not in value:
                for key, val in y_data.items():
                    if key != 'values':
                        val.append(0)
            else:
                for i in xrange(len(value.get('errors', []))):
                    error = value['errors'][i]

                    if 'symerror' in error:
                        error_plus = error['symerror']
                        error_minus = error['symerror']
                    else:
                        error_plus = error['asymerror']['plus']
                        error_minus = error['asymerror']['minus']

                    y_data[error.get('label', 'stat')+'_plus'].append(error_plus)
                    y_data[error.get('label', 'stat')+'_minus'].append(error_minus)

        for entry in y_order:
            data.append(entry)

        for qualifier in dependent_variable.get('qualifiers', []):
            if qualifier['name'] not in qualifiers:
                qualifiers[qualifier['name']] = []
            qualifiers[qualifier['name']].append(qualifier['value'])


# -*- encoding: utf-8 -*-
import os
import StringIO
import hepdata_converter
from hepdata_converter.testsuite import insert_data_as_file, insert_data_as_str, insert_path
from hepdata_converter.testsuite.test_writer import WriterTestSuite

__author__ = 'Michał Szostak'


class YAMLWriterTestSuite(WriterTestSuite):
    @insert_path('oldhepdata/sample.input')
    @insert_data_as_file('oldhepdata/sample.input')
    @insert_data_as_str('oldhepdata/sample.input_single.yaml')
    def test_single_file_output(self, oldhepdata_path, oldhepdata_file, oldhepdata_yaml_file):
        data = hepdata_converter.convert(oldhepdata_file,
                                         options={
                                             'input_format': 'oldhepdata',
                                             'output_format': 'yaml',
                                             'single_file': True})

        yaml_single_output_path = os.path.join(self.current_tmp, 'output.yaml')

        hepdata_converter._main(['--input-format', 'oldhepdata', '--output-format', 'yaml', '--single-file',
                                 oldhepdata_path, yaml_single_output_path])

        self.assertTrue(os.path.exists(yaml_single_output_path))

        self.assertEqual(oldhepdata_yaml_file, data)

        with open(yaml_single_output_path, 'r') as _f:
            self.assertEqual(oldhepdata_yaml_file, _f.read())

    @insert_path('oldhepdata/sample.input')
    @insert_path('oldhepdata/yaml')
    def test_create_newdir(self, oldhepdata_path, yaml_path):
        new_dir = os.path.join(self.current_tmp, 'test', 'dir')

        hepdata_converter.convert(oldhepdata_path, new_dir,
                                  options={'input_format': 'oldhepdata',
                                           'output_format': 'yaml'})

        self.assertTrue(os.path.exists(new_dir))
        self.assertDirsEqual(new_dir, yaml_path)


# -*- coding: utf-8 -*-
import abc
from math import sqrt
import random
import time
from hepdata_converter.writers.array_writer import ArrayWriter, ObjectWrapper, ObjectFactory
import rootpy.io
import rootpy.ROOT
import numpy

__author__ = 'Michał Szostak'


class THFRootClass(ObjectWrapper):
    __metaclass__ = abc.ABCMeta

    _hist_classes = [rootpy.ROOT.TH1F, rootpy.ROOT.TH2F, rootpy.ROOT.TH3F]
    _hist_axes_names = ['x', 'y', 'z']
    dim = 0
    core_object = False

    @classmethod
    def match(cls, independent_variables_map, dependent_variable):
        if not ObjectWrapper.match(independent_variables_map, dependent_variable):
            return False
        elif len(independent_variables_map) == cls.dim:
            for independent_variable in independent_variables_map:
                if 'high' not in independent_variable['values'][0]:
                    break
            else:
                return True
        elif cls.dim == 1 and 'high' in independent_variables_map[0]['values'][0]:
            return True
        return False

    def _create_empty_hist(self, dependant_var_title, index, yval):

        xval = []
        for i in xrange(self.dim):
            xval.append([])
            i_var = self.independent_variables[i]['values']
            for x in i_var:
                xval[i].append(x['low'])
            xval[i].append(i_var[-1]['high'])

        name = "Hist%sD_y%s_e%s" % (self.dim, self.dependent_variable_index + 1, index)

        args = []
        for i in xrange(self.dim):
            args.append(len(xval[i]) - 1)
            args.append(numpy.array(xval[i], dtype=float))

        hist = self._hist_classes[self.dim - 1](self.sanitize_name(name), '', *args)

        for i in xrange(self.dim):
            getattr(hist, self._hist_axes_names[i] + 'axis').title = self.independent_variables[i]['header']['name']

        if self.dim < len(self._hist_classes):
            getattr(hist, self._hist_axes_names[self.dim] + 'axis').title = self.sanitize_name(dependant_var_title)

        for i in xrange(len(yval)):
            hist.fill(*([self.xval[dim_i][i] for dim_i in xrange(self.dim)] + [yval[i]]))
        return hist

    def _create_hist(self, xval):

        name = "Hist%sD_y%s" % (self.dim, self.dependent_variable_index + 1)
        args = []

        for i in xrange(self.dim):
            args.append(len(xval[i]) - 1)
            args.append(numpy.array(xval[i], dtype=float))

        hist = self._hist_classes[self.dim - 1](name, '', *args)

        for i in xrange(self.dim):
            getattr(hist, self._hist_axes_names[i] + 'axis').title = self.independent_variables[i]['header']['name']

        if self.dim < len(self._hist_classes):
            getattr(hist, self._hist_axes_names[self.dim] + 'axis').title = self.dependent_variable['header']['name']

        for i in xrange(len(self.xval[0])):
            hist.fill(*([self.xval[dim_i][i] for dim_i in xrange(self.dim)] + [self.yval[i]]))
        return hist

    def create_objects(self):
        self.calculate_total_errors()

        error_hists = []
        error_labels = {}

        for value in self.dependent_variable.get('values', []):
            for error in value.get('errors', []):
                # :TODO: If error has no label is it stat? Or we have no idea?
                if 'label' not in error:
                    error['label'] = 'stat'
                label = error['label']
                if 'symerror' in error and label not in error_labels:
                    error_labels[label] = 'symerror'
                elif 'asymerror' in error and error_labels.get(label, 'symerror') == 'symerror':
                    error_labels[label] = 'asymerror'

        yvals = []
        index = 1
        for error_label in error_labels:
            if error_labels[error_label] == 'asymerror':
                yval_plus_label = error_label + '_high'
                yval_plus = []
                yval_minus_label = error_label + '_low'
                yval_minus = []

                for value in self.dependent_variable.get('values', []):
                    error = filter(lambda x: x.get('label') == error_label, value.get('errors', []))
                    if len(error) == 0:
                        yval_plus.append(0.0)
                        yval_minus.append(0.0)
                    elif 'symerror' in error[0]:
                        err_val = float(error[0]['symerror'])
                        yval_plus.append(err_val)
                        yval_minus.append(err_val)
                    elif 'asymerror' in error[0]:
                        yval_plus.append(float(error[0]['asymerror']['plus']))
                        yval_minus.append(float(error[0]['asymerror']['minus']))
                    else:
                        yval_plus.append(0.0)
                        yval_minus.append(0.0)

                yvals += [(yval_plus_label, yval_plus, '%s%s' % (index, 'plus')),
                          (yval_minus_label, yval_minus, '%s%s' % (index, 'minus'))]
            else:
                yval = []

                for value in self.dependent_variable.get('values', []):
                    error = filter(lambda x: x.get('label') == error_label, value.get('errors', []))
                    if len(error) == 0:
                        yval.append(0.0)
                    elif 'symerror' in error[0]:
                        yval.append(float(error[0]['symerror']))
                    else:
                        yval.append(0.0)

                yvals += [(error_label, yval, index)]
                index += 1

        for name, vals, index in yvals:
            error_hists.append(self._create_empty_hist(name, index, vals))

        # for i in xrange(self.dim):
        #     self.independent_variable_map.pop(0)

        xval = []
        for i in xrange(self.dim):
            xval.append([])
            i_var = self.independent_variables[i]['values']
            for x in i_var:
                xval[i].append(x['low'])
            xval[i].append(i_var[-1]['high'])

        hist = self._create_hist(xval)

        return [hist] + error_hists


class TH3FRootClass(THFRootClass):
    dim = 3


class TH2FRootClass(THFRootClass):
    dim = 2


class TH1FRootClass(THFRootClass):
    dim = 1


class TGraph2DErrorsClass(ObjectWrapper):

    @classmethod
    def match(cls, independent_variables_map, dependent_variable):
        if not super(TGraph2DErrorsClass, cls).match(independent_variables_map, dependent_variable):
            return False
        if len(independent_variables_map) == 2:
            return True
        return False

    def create_objects(self):
        self.calculate_total_errors()

        self.independent_variable_map.pop(0)
        self.independent_variable_map.pop(0)

        graph = rootpy.ROOT.TGraph2DErrors(len(self.xval[0]),
                                           numpy.array(self.xval[0], dtype=float),
                                           numpy.array(self.xval[1], dtype=float),
                                           numpy.array(self.yval, dtype=float),
                                           numpy.array(self.xerr_plus[0], dtype=float),
                                           numpy.array(self.xerr_plus[1], dtype=float),
                                           numpy.array(self.yerr_plus, dtype=float))

        graph.set_name("Graph2D_y%s" % (self.dependent_variable_index + 1))

        graph.xaxis.title = self.independent_variables[0]['header']['name']
        graph.yaxis.title = self.independent_variables[1]['header']['name']
        graph.zaxis.title = self.dependent_variable['header']['name']

        return [graph]


class TGraphAsymmErrorsRootClass(ObjectWrapper):
    @classmethod
    def match(cls, independent_variables_map, dependent_variable):
        if not super(TGraphAsymmErrorsRootClass, cls).match(independent_variables_map, dependent_variable):
            return False
        if len(independent_variables_map) == 1:
            return True
        return False

    def create_objects(self):
        self.calculate_total_errors()

        self.independent_variable_map.pop(0)

        graph = rootpy.ROOT.TGraphAsymmErrors(len(self.xval[0]),
                                              numpy.array(self.xval[0], dtype=float),
                                              numpy.array(self.yval, dtype=float),
                                              numpy.array(self.xerr_minus[0], dtype=float),
                                              numpy.array(self.xerr_plus[0], dtype=float),
                                              numpy.array(self.yerr_minus, dtype=float),
                                              numpy.array(self.yerr_plus, dtype=float))

        graph.set_name("Graph1D_y%s" % (self.dependent_variable_index + 1))

        graph.xaxis.title = self.independent_variables[0]['header']['name']
        graph.yaxis.title = self.dependent_variable['header']['name']

        return [graph]


class ROOT(ArrayWriter):
    help = 'Writes to ROOT format (binary) converts tables into files containing TH1 objects'
    class_list = [TH3FRootClass, TH2FRootClass, TH1FRootClass, TGraph2DErrorsClass, TGraphAsymmErrorsRootClass]

    def __init__(self, *args, **kwargs):
        super(ROOT, self).__init__(*args, **kwargs)
        self.extension = 'root'

    def _write_table(self, data_out, table):
        data_out.mkdir(table.name)
        data_out.cd(table.name)

        f = ObjectFactory(self.class_list, table.independent_variables, table.dependent_variables)
        for graph in f.get_next_object():
            graph.title = table.name
            graph.write()

    def _prepare_outputs(self, data_out, outputs):
        if isinstance(data_out, str) or isinstance(data_out, unicode):
            self.file_emulation = True
            outputs.append(rootpy.io.root_open(data_out, 'w'))
        # multiple tables - require directory
        elif isinstance(data_out, rootpy.ROOT.TFile):
            outputs.append(data_out)
        else: # assume it's a file like object
            self.file_emulation = True
            outputs.append(rootpy.io.TemporaryFile())

    def write(self, data_in, data_out, *args, **kwargs):
        """

        :param data_in:
        :type data_in: hepconverter.parsers.ParsedData
        :param data_out: filelike object
        :type data_out: file
        :param args:
        :param kwargs:
        """
        self._get_tables(data_in)

        self.file_emulation = False
        outputs = []
        self._prepare_outputs(data_out, outputs)
        output = outputs[0]

        for i in xrange(len(self.tables)):
            table = self.tables[i]

            self._write_table(output, table)

        if data_out != output and hasattr(data_out, 'write'):
            output.flush()
            output.re_open('read')
            buff = bytearray(output.get_size())
            output.read_buffer(buff, output.get_size())
            data_out.write(buff)

        if self.file_emulation:
            output.close()


import unittest
from hepdata_converter.parsers import Parser
from hepdata_converter.parsers.oldhepdata_parser import OldHEPData


class ParserTestSuite(unittest.TestCase):
    """Test suite for Parser factory class
    """

    def test_get_specific_parser_oldhepdata(self):
        self.assertEqual(Parser.get_concrete_class('oldhepdata').__class__, OldHEPData.__class__)

    def test_get_specific_parser_nonexist(self):
        self.assertRaises(ValueError, Parser.get_concrete_class, 'nonexisting_parser')


import unittest
import os
import tempfile
import shutil
import time
from hepdata_converter import convert
from hepdata_converter.testsuite import insert_path
from hepdata_converter.testsuite.test_writer import WriterTestSuite


class ArrayWriterTestSuite(WriterTestSuite):
    @insert_path('yaml_full')
    def test_select_table(self, submission_filepath):
        csv_content = convert(submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': os.path.join(submission_filepath, 'data1.yaml')})

        csv_content = convert(submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 'Table 1'})

        csv_content = convert(submission_filepath, options={'input_format': 'yaml',
                                                                 'output_format': 'csv',
                                                                 'table': 0})



import StringIO
import argparse
import sys
import version
from parsers import Parser
from writers import Writer


def convert(input, output=None, options={}):
    """Converts arbitrary supported data format to the new HEPData YAML format, and writes the output files
    to the output_dir directory

    :param input: input, depending on the choosen input datatype it may be, filepath, filetype object, directory, etc
    :param output: output directory to which converted YAML files will be written
    :param input_format: format of the input data, must be a string containing name of the input parser class
    :type input_format: str
    :param options: additional options used for conversion (depends on the choosen input format)
    :type options: dict
    :raise ValueError: raised if no input_format is specified
    """

    if 'input_format' not in options and 'output_format' not in options:
        raise ValueError("no input_format and output_format specified!")

    input_format = options.get('input_format', 'yaml')
    output_format = options.get('output_format', 'yaml')

    parser = Parser.get_concrete_class(input_format)(**options)
    writer = Writer.get_concrete_class(output_format)(**options)

    if not output and not writer.single_file_output:
        raise ValueError("this output_format requires specifing 'output' argument")

    # if no output was specified create proxy output to which writer can insert data
    _output = output
    if not _output:
        _output = StringIO.StringIO()

    writer.write(parser.parse(input), _output)

    # if no output was specified return output
    if not output:
        return _output.getvalue()


def make_exit(message='', code=0):
    return code, message


def generate_help_epilogue():
    margin = '   '

    r = 'Parsers:\n'
    r += '[use them as --input-format parameter]\n'
    r += '\n'

    for cls in Parser.get_all_subclasses():
        r += cls.get_help(margin)

    r += '\nWriters:\n'
    r += '[use them as --output-format parameter]\n'
    r += '\n'

    for cls in Writer.get_all_subclasses():
        r += cls.get_help(margin)

    return r


def _main(arguments=sys.argv):
    # if version is specified ignore any other arguments
    if '--version' in arguments or '-v' in arguments:
        return make_exit(message="hepdata-converter version: %s" % version.__version__)

    parser = argparse.ArgumentParser(description="CLI tools for converting between HEP data formats", add_help=True,
                                     formatter_class=argparse.RawTextHelpFormatter,
                                     epilog=generate_help_epilogue())
    parser.add_argument("--input-format", '-i', action='store', default='yaml', help='format of the input file/s (default: yaml) [choose one option from Parsers section below]')
    parser.add_argument("--output-format", '-o', action='store', default='yaml', help='format of the output file/s (default: yaml) [choose one option from Writers section below]')
    parser.add_argument("--version", '-v', action='store_const', const=True, default=False, help='Show hepdata-converter version')
    parser.add_argument("input")
    parser.add_argument("output")

    if arguments == sys.argv:
        arguments = sys.argv[1:]

    program_args = vars(parser.parse_known_args(arguments)[0])

    input_format = program_args['input_format']
    output_format = program_args['output_format']

    Parser.get_concrete_class(input_format).register_cli_options(parser)
    Writer.get_concrete_class(output_format).register_cli_options(parser)

    # reparse arguments, now with added options from concrete parsers / writers
    program_args = vars(parser.parse_args(arguments))
    try:
        convert(program_args['input'], program_args['output'], program_args)
        return make_exit()
    except ValueError as e:
        return make_exit(message="Options error: %s" % str(e), code=1)


def main(arguments=sys.argv):
    r, message = _main(arguments)
    if r == 0:
        print message
    else:
        print >> sys.stderr, message
    sys.exit(r)



# -*- encoding: utf-8 -*-
import os
import StringIO
import rootpy
import hepdata_converter
from hepdata_converter.testsuite import insert_data_as_file, insert_path, insert_data_as_str
from hepdata_converter.testsuite.test_writer import WriterTestSuite
from rootpy.io import root_open

__author__ = 'Michał Szostak'


class YODAWriterTestSuite(WriterTestSuite):
    
    @insert_path('yaml_full')
    @insert_data_as_file('yoda/full.yoda')
    def test_simple_parse(self, yaml_simple_path, yoda_template):
        output_file_path = os.path.join(self.current_tmp, 'datafile.yoda')
        hepdata_converter.convert(yaml_simple_path, output_file_path,
                                  options={'output_format': 'yoda'})

        self.assertNotEqual(os.stat(output_file_path).st_size, 0, 'output yoda file is empty')
        with open(output_file_path, 'r') as f:
            self.assertMultiLineAlmostEqual(f, yoda_template)


import copy
import csv
import os
from hepdata_converter.common import OptionInitMixin, Option
from hepdata_converter.writers.array_writer import ArrayWriter


class CSV(ArrayWriter):
    help = 'Writes to CSV format, it can write either one table (specified by --table parameter) or all tables from the ' \
           'input file. In the case of one table output must be filepath to the new csv file, in the case of multiple tables ' \
           'the output must be specified to be a directory to which all table files should be written'

    @classmethod
    def options(cls):
        options = ArrayWriter.options()
        options['pack'] = Option('pack', type=bool, default=False, required=False,
                                 help=('If specified, dependand variables will be put in one table, instead of creating one '
                                       'table per dependant variable in CSV file'))
        options['separator'] = Option('separator', type=str, default=',', required=False,
                                      help='Defines separator for CSV file, the default is colon: ":"')
        
        return options

    def __init__(self, *args, **kwargs):
        super(CSV, self).__init__(*args, **kwargs)
        self.extension = 'csv'

    def _write_metadata(self, data_out, table):
        data_out.write(unicode("#: name: %s\n" % table.metadata['name']).encode('utf8', 'replace'))
        data_out.write(unicode("#: description: %s\n" % table.metadata['description']).encode('utf8', 'replace'))
        data_out.write(unicode("#: data_file: %s\n" % table.metadata['data_file']).encode('utf8', 'replace'))

        #license:
        if 'data_license' in table.metadata and table.metadata['data_license']:
            license_text = (table.metadata['data_license'].get('name') or '') + ' ' + \
                           (table.metadata['data_license'].get('url') or '') + ' ' + \
                           (table.metadata['data_license'].get('description') or '')

            data_out.write("#: data_license: %s\n" % license_text)

        for keyword in table.metadata.get('keywords', []):
            data_out.write("#: keyword %s: %s\n" % (keyword['name'], ' | '.join([str(val) for val in keyword.get('values', [])])))

    def _write_table(self, data_out, table):
        if self.pack:
            self._write_packed_data(data_out, table)
        else:
            self._write_unpacked_data(data_out, table)

    def _write_csv_data(self, output, qualifiers, qualifiers_marks, headers, data):

            lineterminator = '\n'
            self._write_qualifiers(output, qualifiers, qualifiers_marks, self.separator, lineterminator)

            csv_writer = csv.writer(output, delimiter=self.separator, lineterminator=lineterminator, quotechar="'", quoting=csv.QUOTE_NONNUMERIC)
            csv_writer.writerow(headers)

            for i in xrange(len(data[0])):
                csv_writer.writerow([data[j][i] for j in xrange(len(data)) ])

            return csv_writer

    @classmethod
    def _write_qualifiers(cls, writer, qualifiers, qualifiers_marks, field_separator, newline):
        for qualifier_key in qualifiers:
            row = []
            i = 0
            for qualifier in qualifiers[qualifier_key]:
                for i in xrange(i, len(qualifiers_marks)):
                    if qualifiers_marks[i]:
                        row.append(qualifier)
                        i += 1
                        break
                    else:
                        row.append(None)
            writer.write(unicode(field_separator.join([str(val) if val is not None else '' for val in
                                                       ['#: ' + qualifier_key] + row]) + newline).replace('utf8', 'replace'))

    def _write_packed_data(self, data_out, table):
        """This is kind of legacy function - this functionality may be useful for some people, so even though
        now the default of writing CSV is writing unpacked data (divided by independent variable) this method is
        still available and accesable if ```pack``` flag is specified in Writer's options

        :param output: output file like object to which data will be written
        :param table: input table
        :type table: hepdata_converter.parsers.Table
        """
        headers = []
        data = []
        qualifiers_marks = []
        qualifiers = {}

        self._extract_independent_variables(table, headers, data, qualifiers_marks)

        for dependent_variable in table.dependent_variables:
            self._parse_dependent_variable(dependent_variable, headers, qualifiers, qualifiers_marks, data)

        self._write_metadata(data_out, table)
        self._write_csv_data(data_out, qualifiers, qualifiers_marks, headers, data)

    def _write_unpacked_data(self, output, table):
        headers_original = []
        data_original = []
        qualifiers_marks_original = []

        self._extract_independent_variables(table, headers_original, data_original, qualifiers_marks_original)

        self._write_metadata(output, table)

        for dependent_variable in table.dependent_variables:
            qualifiers = {}
            # make a copy of the original list
            headers = list(headers_original)
            data = list(data_original)
            qualifiers_marks = copy.deepcopy(qualifiers_marks_original)
            self._parse_dependent_variable(dependent_variable, headers, qualifiers, qualifiers_marks, data)
            self._write_csv_data(output, qualifiers, qualifiers_marks, headers, data)
            output.write('\n')


import unittest
import os
import tempfile
import shutil
import time
import yaml
from hepdata_converter.testsuite import TMPDirMixin, ExtendedTestCase
from hepdata_converter.writers import Writer


class WriterTestSuite(TMPDirMixin, ExtendedTestCase):
    pass




# -*- coding: utf-8 -*-
import os
from setuptools import setup
import re


def get_all_datafiles(package, path):
    r = []
    setup_path = os.path.dirname(__file__)
    for abs_path, dirs, files in os.walk(os.path.join(setup_path, package, path)):
        r += [os.path.join(path, f) for f in files]
    return r


def get_version():
    with open('hepdata_converter/version.py', 'r') as version_f:
        content = version_f.read()

    r = re.search('^__version__ *= *\'(?P<version>.+)\'', content, flags=re.MULTILINE)
    if not r:
        return '0.0.0'
    return r.group('version')

setup(
    name='hepdata-converter',
    version=get_version(),
    install_requires=[
        'pyyaml',
        'hepdata_validator',
        'rootpy',
        'numpy',
        'matplotlib'
    ],
    entry_points={
        'console_scripts': [
            'hepdata-converter = hepdata_converter:main',
        ]
    },

    packages=['hepdata_converter', 'hepdata_converter.parsers', 'hepdata_converter.writers', 'hepdata_converter.testsuite'],
    package_data={'hepdata_converter': get_all_datafiles(package='hepdata_converter/testsuite', path='testdata')},
    include_package_data=True,

    url='https://github.com/HEPData/hepdata-converter/',
    license='GPL',
    author='Michał Szostak',
    author_email='michal.florian.szostak@cern.ch',
    description='Library providing means of conversion between oldhepdata format to new one, and new one to csv / yoda / root etc.',
    download_url='https://github.com/HEPData/hepdata-converter/tarball/%s' % get_version(),
)


import copy
import numpy
from hepdata_converter.writers.array_writer import ArrayWriter, ObjectWrapper, ObjectFactory
import yoda


class ScatterYodaClass(ObjectWrapper):
    dim = 0
    _scatter_classes = [yoda.core.Scatter2D, yoda.core.Scatter3D]
    _point_classes = [yoda.core.Point2D, yoda.core.Point3D]

    @classmethod
    def get_scatter_cls(cls):
        return cls._scatter_classes[cls.dim - 1]

    @classmethod
    def get_point_cls(cls):
        return cls._point_classes[cls.dim - 1]

    @classmethod
    def match(cls, independent_variables_map, dependent_variable):
        if not ObjectWrapper.match(independent_variables_map, dependent_variable):
            return False
        elif len(independent_variables_map) == cls.dim or cls.dim == len(cls._scatter_classes) - 1:
            return True
        return False

    def _create_scatter(self, xval):
        graph = self.get_scatter_cls()()

        for i in xrange(len(self.yval)):
            args = []

            for dim_i in xrange(self.dim):
                args.append(self.xval[dim_i][i])
            args.append(self.yval[i])
            for dim_i in xrange(self.dim):
                args.append(self.xerr_plus[dim_i][i])
            args.append(self.yerr_plus[i])

            graph.addPoint(self.get_point_cls()(*args))
        return graph

    def create_objects(self):
        self.calculate_total_errors()

        for i in xrange(self.dim):
            self.independent_variable_map.pop(0)

        graph = self._create_scatter(self.xval)

        return [graph]


class Scatter2DYodaClass(ScatterYodaClass):
    dim = 1


class Scatter3DYodaClass(ScatterYodaClass):
    dim = 2


class YODA(ArrayWriter):
    help = 'Writes YODA output for table specified by --table parameter, the output should be defined as ' \
           'filepath to output yoda file'

    class_list = [Scatter3DYodaClass, Scatter2DYodaClass]

    def __init__(self, *args, **kwargs):
        super(YODA, self).__init__(*args, **kwargs)
        self.extension = 'yoda'

    def _prepare_outputs(self, data_out, outputs):
        if isinstance(data_out, str) or isinstance(data_out, unicode):
            self.file_emulation = True
            outputs.append(open(data_out, 'w'))
        # multiple tables - require directory
        else:  # assume it's a file like object
            self.file_emulation = True
            outputs.append(data_out)

    def _write_table(self, data_out, table):
        # headers_original = []
        # qualifiers_marks_original = []
        f = ObjectFactory(self.class_list, table.independent_variables, table.dependent_variables)
        for graph in f.get_next_object():
            graph.title = table.name
            graph.path = ''
            yoda.core.writeYODA(graph, data_out)

    def write(self, data_in, data_out, *args, **kwargs):
        """

        :param data_in:
        :type data_in: hepconverter.parsers.ParsedData
        :param data_out: filelike object
        :type data_out: file
        :param args:
        :param kwargs:
        """
        self._get_tables(data_in)

        self.file_emulation = False
        outputs = []
        self._prepare_outputs(data_out, outputs)
        output = outputs[0]

        for i in xrange(len(self.tables)):
            table = self.tables[i]

            self._write_table(output, table)

        if self.file_emulation:
            output.close()


# -*- encoding: utf-8 -*-
import os
import StringIO
import rootpy
import hepdata_converter
from hepdata_converter.testsuite import insert_data_as_file, insert_path, insert_data_as_str
from hepdata_converter.testsuite.test_writer import WriterTestSuite
from rootpy.io import root_open

__author__ = 'Michał Szostak'
import ROOT as _ROOT
import unittest
from hepdata_converter.writers.root_writer import ROOT


class ROOTWriterTestSuite(WriterTestSuite):
    
    @insert_path('yaml_full')
    @insert_path('root/full.root')
    def test_simple_parse(self, yaml_full_path, full_root_path):
        output_file_path = os.path.join(self.current_tmp, 'datafile.root')
        hepdata_converter.convert(yaml_full_path, output_file_path,
                                  options={'output_format': 'root'})

        self.assertNotEqual(os.stat(output_file_path).st_size, 0, 'output root file is empty')

        with root_open(output_file_path, 'r') as f, root_open(full_root_path, 'r') as f_orig:
            self.assertEqual(list(f.walk()), list(f_orig.walk()))

        with open(output_file_path, 'w') as output:
            hepdata_converter.convert(yaml_full_path, output,
                                      options={'output_format': 'root'})

        self.assertNotEqual(os.stat(output_file_path).st_size, 0, 'output root file is empty')

        with root_open(output_file_path, 'r') as f, root_open(full_root_path, 'r') as f_orig:
            self.assertEqual(list(f.walk()), list(f_orig.walk()))
            for path, dirs, objects in list(f.walk())[1:]:
                for object in objects:
                    o = f.get('%s/%s' % (path, object))
                    o_orig = f_orig.get('%s/%s' % (path, object))
                    self.assertEqual(o.__class__, o_orig.__class__)
                    if o.__class__.__name__ == rootpy.plotting.graph.Graph.__name__:
                        self.assertEqual(list(o.x()), list(o_orig.x()))
                        self.assertEqual(list(o.y()), list(o_orig.y()))

    @insert_path('yaml_full')
    @insert_path('root/full.root')
    def test_th1_parse(self, yaml_full_path, full_root_path):
        output_file_path = os.path.join(self.current_tmp, 'datafile.root')
        hepdata_converter.convert(yaml_full_path, output_file_path,
                                  options={'output_format': 'root', 'table': 'data2.yaml'})
        pass



import yaml
from hepdata_validator.submission_file_validator import SubmissionFileValidator
from hepdata_validator.data_file_validator import DataFileValidator
from hepdata_converter.parsers import Parser, ParsedData, Table
import os


class YAML(Parser):
    help = 'Parses New HEPData YAML format. Input parameter should be path to the directory where submission.yaml file ' \
           'is present (or direct filepath to the submission.yaml file)'

    def __init__(self, *args, **kwargs):
        super(YAML, self).__init__(*args, **kwargs)

    def _pretty_print_errors(self, message_dict):
        return ' '.join(['%s: %s' % (key, ' | '.join([e.message for e in val])) for key, val in message_dict.items()])

    def parse(self, data_in, *args, **kwargs):
        """

        :param data_in: path to submission.yaml
        :param args:
        :param kwargs:
        :raise ValueError:
        """
        if not os.path.exists(data_in):
            raise ValueError("File / Directory does not exist: %s" % data_in)

        if os.path.isdir(data_in):
            submission_filepath = os.path.join(data_in, 'submission.yaml')
            if not os.path.exists(submission_filepath):
                submission_filepath = os.path.join(data_in, 'submission.yml')
                if not os.path.exists(submission_filepath):
                    raise ValueError("No submission file in %s" % data_in)
            data_in = submission_filepath

        # first validate submission file:
        submission_file_validator = SubmissionFileValidator()
        if not submission_file_validator.validate(data_in):
            raise RuntimeError("Submission file (%s) did not pass validation: %s" %
                               (data_in, self._pretty_print_errors(submission_file_validator.get_messages())))

        with open(data_in, 'r') as submission_file:
            submission_data = list(yaml.load_all(submission_file))

        if len(submission_data) == 0:
            raise RuntimeError("Submission file (%s) is empty" % data_in)

        tables = []

        # validator for table data
        data_file_validator = DataFileValidator()

        for i in range(1, len(submission_data)):
            table_filepath = os.path.join(os.path.dirname(data_in), submission_data[i]['data_file'])
            if not os.path.exists(table_filepath):
                raise ValueError("table file: %s does not exist" % table.data_file)

            # validate table file:
            if not data_file_validator.validate(table_filepath):
                raise RuntimeError("Data file (%s) did not pass validation: %s" %
                                   (table_filepath, self._pretty_print_errors(data_file_validator.get_messages())))

            with open(table_filepath, 'r') as table_file:
                table_data = yaml.load(table_file)

            table = Table(index=i, metadata=submission_data[i], data=table_data)
            tables.append(table)

        return ParsedData(submission_data[0], tables)
